//: /be/optiloading/Ship.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.ArrayList;

/** 
 * Ship class used to stow information about the ship and do ship's calculations.
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank" href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Ship {
	
	// Basic ship's parameters
	private float lightship;
	private float lightshipLCG;
	private float lightshipKG;
	private float lbpp;
	private float displacement;
	private float mindisplacement;
	private float maxdisplacement;
	private float maxvolume;
	private float draught;
	private float deadweight;
	private float constants;
	private float lcf;
	private float lcb;
	private float lcg;
	private float lcgtemp;
	private float mct;
	private float km;
	private float bm;
	private float gm;
	private float kg;
	private float kb;
	private float fsc;
	private float trim;
	private float dfwd;
	private float daft;
	private float density;
	private float constvert;
	private float constlong;
	private float constfsm;
	
	/**
	 * An ArrayList containing the hydrostatic table. 
	 * This ArrayList if filled within the loadHydrostaticTables(Inputstream hydrostatictable).
	 */
	private ArrayList<HydrostaticData> data = new ArrayList<HydrostaticData>();
	
	/**
	 * An ArrayList containing the section data (frames) used to make buoyancy calculations.
	 * This ArrayList if filled within the loadBuoyancyData(InputStream buoyancyd).
	 */
	private ArrayList<Section> sectionlist = new ArrayList<Section>();
	
	/**
	 * An ArrayList containing the intensity data used to make hull stress calculations.
	 * This ArrayList if filled within the loadIntensityData(InputStream lightshipd).
	 */
	public ArrayList<Intensity> intensitylist;
	
	// ArrayList which stores percentages of the total buoyancy force for each section
	private ArrayList<Float> buoyancydistribution;
	
	
	/**
	 * Represents a list of all the ship's cargo tanks.
	 * This list contains Tank objects.
	 */
	public ArrayList<Tank> tanks = new ArrayList<Tank>();
	
	//Contains the Cargo obejct which is to be loaded.
	public Cargo toLoad;
	
	// A unique instance of the Ship object created in the private constructor (singleton).
	private static Ship uniqueInstance;
	
	// Private constructor which is only calles from the getInstance() when there is no uniqueInstance.
	private Ship(){
		// Load the ship data
		loadShipData(this.getClass().getResourceAsStream("/data/shipdata.dat"),  //General data
					 this.getClass().getResourceAsStream("/data/hydrostatictables.dat"),  //Hydrostatic tables
					 this.getClass().getResourceAsStream("/data/tankdata.dat"), //Ullage tables
					 this.getClass().getResourceAsStream("/data/buoyancy.dat")); //Buoyancy section
	}
	
	/**
	 * Returns the uniqueInstance of the Ship object (Singleton).
	 * @return Ship object.
	 */
	public static Ship getInstance(){
		
		// Create a new Ship object when no unique instance is in existence.
		if (uniqueInstance == null){
			uniqueInstance = new Ship();
		}
		return uniqueInstance;
	}
	
	/**
	 * Method to load all the ship's data.
	 * @param shipd InputStream object containing general ship's data.
	 * @param hydrod InputStream object containing the ship's hydrostatic tables.
	 * @param tankd InputStream object containg tank data from the ship.
	 * @param buoyancyd InputStream object containg section data from the ship.
	 */
	public void loadShipData(InputStream shipd, InputStream hydrod, InputStream tankd, InputStream buoyancyd){
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(shipd)); //Get a reader to read the file
			String line = reader.readLine(); //Read the first line
			String[] row = line.split(";"); // Split the string into an array of Strings
			
			//Assign each value (general ship information)
			lbpp = Float.parseFloat(row[0]);
			lightship = Float.parseFloat(row[1]);
			lightshipLCG = Float.parseFloat(row[2]);
			lightshipKG = Float.parseFloat(row[3]);
			mindisplacement = Float.parseFloat(row[4]);
			maxdisplacement = Float.parseFloat(row[5]);
			maxvolume = Float.parseFloat(row[6]);
			
			reader.close(); //Close the file
			
		} catch(Exception ex) {
			System.out.println("Cannot read file");
		}
		
		loadTankData(tankd); //Call the private method loadTankdata(File tankdata)
		loadHydrostaticTable(hydrod); //Call the private method loadHydrostaticTables(File hydrostatictable)
		loadBuoyancyData(buoyancyd); //Call the private method loadBuoyancyData(buoyancyd)
	}
	
	
	// Method to load the hydrostatic table
	private void loadHydrostaticTable(InputStream hydrostatictable){
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(hydrostatictable)); //Get a reader to read the file
			String line = null;
			
			//Assign each line to the String line until line equals null.
			while((line=reader.readLine()) != null){
				String[] row = line.split(";"); //Split the sting into an array of Strigs
				
				//Create a new HydrostaticData object stored in the private Arraylist<HydrostaticData> data
				data.add(new HydrostaticData(Float.parseFloat(row[0]),
											 Float.parseFloat(row[1]),
											 Float.parseFloat(row[2]),
											 Float.parseFloat(row[3]),
										     Float.parseFloat(row[4]),
											 Float.parseFloat(row[5]),
											 Float.parseFloat(row[6])));
			}
			reader.close(); //Close the file
			
		} catch(Exception ex) {
			System.out.println("Cannot read hydro file");
		}
	}
	
	// Method to load the data of each tank and stow them as Tank Objects in the public ArrayList<Tank> tanks.
	private void loadTankData(InputStream tankdata){
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(tankdata)); //Get a reader to read the file
			Tank currentTank = null;
			String line = null;
			
			//Run trough the file and create a new Tank when the line starts with "N".
			//Stow the tankdata of each tank in the object.
			//Stow the Tank object in the public ArrayList<Tank> tanks.
			while((line=reader.readLine()) != null){
				String[] row = line.split(";");
				if (row[0].equals("N")){
					currentTank = new Tank(row[1],Float.parseFloat(row[2]),Float.parseFloat(row[3]),Integer.parseInt(row[4]),Integer.parseInt(row[5]));
					tanks.add(currentTank);
				} else {
					currentTank.addData(Float.parseFloat(row[0]), Float.parseFloat(row[1]), Float.parseFloat(row[2]), Float.parseFloat(row[3]));
				}
			}
			reader.close(); //Close the file
		} catch(Exception ex) {
			System.out.println("Cannot read tank file");
		}
	}
	
	//Method to load the data of each section in the private sectionlist ArrayList<Section>.
	private void loadBuoyancyData(InputStream buoyancyd){
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(buoyancyd)); //Get a reader to read the file
			String line = null;

			while((line=reader.readLine()) != null){
				String[] row = line.split(";"); //Split the sting into an array of Strigs

				sectionlist.add(new Section(Float.parseFloat(row[0]),
											Float.parseFloat(row[1])));
				}
				reader.close(); //Close the file
			} catch(Exception ex) {
				System.out.println("Cannot read buoyancy file");
			}
	}
	
	//Method to load the intensity of the light ship into the intensitylist ArrayList<Intensity>.
	private void loadIntensityData(InputStream lightshipd){
		try{
			BufferedReader reader = new BufferedReader(new InputStreamReader(lightshipd)); //Get a reader to read the file
			String line = null;

			while((line=reader.readLine()) != null){
				String[] row = line.split(";"); //Split the sting into an array of Strigs

				intensitylist.add(new Intensity(Float.parseFloat(row[0]),
												Float.parseFloat(row[1])));
				}
				reader.close(); //Close the file
			} catch(Exception ex) {
				System.out.println("Cannot read misc file");
			}
	}
	
	//Private method to calculate the total moment for a specific buoyancy distribution
	private float calculateTotalmoment(){
		float returnvalue = 0;
		for(int i = 0; i < buoyancydistribution.size(); i++){
			returnvalue += (buoyancydistribution.get(i)*displacement*sectionlist.get(i).getLCB());
		}
		return returnvalue;
	}
	
	//Private method to find a proper buoyancy distribution for a given LCB
	private void calculateBuoyancyDistribution(float trimtemp, ArrayList<Float> constants){
		buoyancydistribution = new ArrayList<Float>(); //Create a new buoyancydistribution list
		
		//Load the list with the indication values (buoyancy equally distributed)
		for(Section x: sectionlist){
			buoyancydistribution.add(x.getIndication());
		}
		float totalmoment = calculateTotalmoment(); //Calculate the total moment
		float target = displacement * lcb; //Calculate the target moment given the LCB
		boolean withinrange = false; //Initialize boolean value to indicate the given moment is within range of the target

		//Evenkeel, change the moment to come within range of the target
		while(withinrange == false){
			//Aft section
			buoyancydistribution.set(0, (float) (buoyancydistribution.get(0) - (107.5*lcb/100000000)));
			buoyancydistribution.set(1, (float) (buoyancydistribution.get(1) - (105*lcb/100000000)));
			buoyancydistribution.set(2, (float) (buoyancydistribution.get(2) - (102.5*lcb/100000000)));
			buoyancydistribution.set(3, (float) (buoyancydistribution.get(3) - (100*lcb/100000000)));
			buoyancydistribution.set(4, (float) (buoyancydistribution.get(4) - (97.5*lcb/100000000)));
			buoyancydistribution.set(5, (float) (buoyancydistribution.get(5) - (95*lcb/100000000)));
			buoyancydistribution.set(6, (float) (buoyancydistribution.get(6) - (92.5*lcb/100000000)));
			buoyancydistribution.set(7, (float) (buoyancydistribution.get(7) - (90*lcb/100000000)));
			
			//Mid section
			for(int i= 8; i<16; i++){
				buoyancydistribution.set(i, (float) (buoyancydistribution.get(i) + (7.5*lcb/100000000)));
			}
			
			//Fwd section
			buoyancydistribution.set(16, (float) (buoyancydistribution.get(16) + (50*lcb/100000000)));
			buoyancydistribution.set(17, (float) (buoyancydistribution.get(17) + (75*lcb/100000000)));
			buoyancydistribution.set(18, (float) (buoyancydistribution.get(18) + (80*lcb/100000000)));
			buoyancydistribution.set(19, (float) (buoyancydistribution.get(19) + (75*lcb/100000000)));
			buoyancydistribution.set(20, (float) (buoyancydistribution.get(20) + (90*lcb/100000000)));
			buoyancydistribution.set(21, (float) (buoyancydistribution.get(21) + (110*lcb/100000000)));
			buoyancydistribution.set(22, (float) (buoyancydistribution.get(22) + (120*lcb/100000000)));
			buoyancydistribution.set(23, (float) (buoyancydistribution.get(23) + (130*lcb/100000000)));

			totalmoment = calculateTotalmoment();
			
			if(totalmoment > (target-50)){
				withinrange = true;
			}
		}
		
		//Trim correction
		lcgtemp = lcb - ((trimtemp*mct)/displacement); //Calculate the lcg for a desired trim
		if(lcb != lcgtemp){
			target = displacement * lcgtemp;
			withinrange = false;
			
			while(withinrange == false){
				
				//Trim by the stern
				if(lcb > lcgtemp){
					for(int i = 0; i <12; i++){
						buoyancydistribution.set(i, (float) (buoyancydistribution.get(i) + ((12-i)*0.0000001)));
					}
					for(int i = 12; i <24; i++){
						buoyancydistribution.set(i, (float) (buoyancydistribution.get(i) - ((i-11)*0.0000001)));
					}
					totalmoment = calculateTotalmoment();

					if(totalmoment < (target+100)){
						withinrange = true;
					}
				} else { 
					//Trim by the bow
					for(int i = 0; i <12; i++){
						buoyancydistribution.set(i, (float) (buoyancydistribution.get(i) - ((12-i)*0.0000001)));
					}
					for(int i = 12; i <24; i++){
						buoyancydistribution.set(i, (float) (buoyancydistribution.get(i) + ((i-11)*0.0000001)));
					}
					totalmoment = calculateTotalmoment();
					
					if(totalmoment > (target-100)){
						withinrange = true;
					}
				}
			}
		}
		
		//Looad the light ship intensitylist
		intensitylist = new ArrayList<Intensity>();
		loadIntensityData(this.getClass().getResourceAsStream("/data/lightship.dat"));
		
		//Add the buoyancy forces to the light ship intensitylist
		int j = 0;
		for(int i = 1; i < 36; i++){
			float in = (buoyancydistribution.get(j)*displacement)/4;
			intensitylist.get(i).addIntensity(in);
			if((i % 5)==0){
				j++;
			}
		}
		for(int i = 36; i <41; i++){
			float in = (float) ((buoyancydistribution.get(j)*displacement)/4.1);

			intensitylist.get(i).addIntensity(in);
		}
		j++;
		for(int i = 41; i < 217; i++){
			float in = (float) ((buoyancydistribution.get(j)*displacement)/19.8);
			intensitylist.get(i).addIntensity(in);
			if(((i-40) % 22)==0){
				j++;
			}
		}
		for(int i = 217; i < 245; i++){
			float in = (float) ((buoyancydistribution.get(j)*displacement)/3.6);
			intensitylist.get(i).addIntensity(in);
			if(((i-216) % 4)==0){
				j++;
			}
		}
		for(int i = 245; i < 249; i++){
			float in = (float) (buoyancydistribution.get(j)*displacement)/3;
			intensitylist.get(i).addIntensity(in);
		}
		
		//Add the forces created by the miscellaneous compartments.
		for(int i = 0; i < intensitylist.size();i++){
			float in = constants.get(i);
			intensitylist.get(i).addIntensity(in);
		}
		
	}
	
	/**
	 * Method to check the cargo and set the to be loaded cargo if the desired cargo value is acceptable.
	 * @param tload Cargo object containing the weight, volume and specific gravity of the cargo.
	 * @param dens Floating point value that contains the density of the water in which the ship is floating.
	 * @param f Floating point value that contains the desired trim.
	 * @param c Floating point value that contains the weight of the miscellaneous compartments.
	 * @return Boolean value is true is cargo is valid else false.
	 */
	public boolean checkToLoad(Cargo tload, float dens, float t,float c){
		boolean validload = false;	//return value; changed if cargo is valid
		
		// The max and minimum displacement changes when the ship isn't floating in sea water
		// Because it is not yet certain the tload value is acceptable this is a temporary value
		float tempmindisplacement = (float) (mindisplacement * (dens/1.025));
		float tempmaxdisplacement = (float) (maxdisplacement * (dens/1.025));
		
		float wanteddisplacement = lightship + c + tload.getWeight(); //wanteddisplacement = light ship + deadweight
		
		// Check is the displacement is greater then the minimun displacement and lesser then the max displacement
		// Check if the volume does not exceed the maximum volume
		if ((tempmindisplacement <= wanteddisplacement) && 
		    (wanteddisplacement <= tempmaxdisplacement) &&
		    ((tload.getVolume()) <= maxvolume)){
			displacement = lightship + tload.getWeight() + c; //set the displacement
			toLoad = tload; // set the to be loaded Cargo object
			mindisplacement = tempmindisplacement; // set the min displacement
			maxdisplacement = tempmaxdisplacement; // set the max displacement
			constants = c; //Set the weight of the miscellaneous compartments
			deadweight = displacement-lightship; //Calculate and set the deadweight
			calculateInitial(dens); // calculate the value of the draught, lcf, lcb and mct for the given displacement
			
			//Updat the GUI
			Gui optiloading = Gui.getInstance();
			optiloading.misccompartments.loadTanks();
			optiloading.setConstants(displacement, deadweight);
			
			calculateBuoyancyDistribution(t,optiloading.misccompartments.getIntensityList());//Calculate buoyancy distribution
			
			//Get the moments created by the miscellaneous compartments.
			ArrayList<Float> constantvalues = optiloading.misccompartments.getMoments();
			constlong = constantvalues.get(0);
			constvert = constantvalues.get(1);
			constfsm = constantvalues.get(2);
			
			optiloading = null; //release the GUI object
			validload = true;  // change the return value to true
		}
		return validload;
	}
	
	/**
	 * Method to check if it is possible to achieve a desired TMD and if so set the to be loaded cargo value.
	 * @param tmd Floating point value indicating the desired TMD.
	 * @param spec Floating point value indicating the specific gravity of the cargo to be loaded.
	 * @param dens Floating point value that contains the density of the water in which the ship is floating.
	 * @param f Floating point value that contains the desired trim.
	 * @param c Floating point value that contains the weight of the miscellaneous compartments.
	 * @return Boolean value is true is cargo is valid else false.
	 */
	public boolean checkToLoad(float tmd,float spec, float dens, float t,float c){
		boolean validload = false; //return value; changed if cargo is valid
		
		// The max and minimum displacement changes when the ship isn't floating in sea water
		// Because it is not yet certain the tload value is acceptable this is a temporary value
		float tempmindisplacement = (float) (mindisplacement * (dens/1.025));
		float tempmaxdisplacement = (float) (maxdisplacement * (dens/1.025));
		
		// Get a to be loaded cargo value if the desired TMD is to be achieved using the method getDispAtTMD(tmd,dens)
		Cargo tload = new Cargo(getDispAtTMD(tmd,dens)-lightship-c,spec);	
		
		float wanteddisplacement = lightship + c + tload.getWeight();
		// Check is the displacement is greater then the minimun displacement and lesser then the max displacement
		// Check if the volume does not exceed the maximum volume
		if ((tempmindisplacement <= wanteddisplacement) && 
			(wanteddisplacement <= tempmaxdisplacement) &&
			((tload.getVolume()) <= maxvolume)){
			displacement = lightship + tload.getWeight() + c; // set the displacement
			toLoad = tload; // set the to be loaded Cargo object
			mindisplacement = tempmindisplacement; // set the min displacement
			maxdisplacement = tempmaxdisplacement; // set the max displacement
			constants = c; //Set the weight of the miscellaneous compartments
			deadweight = displacement-lightship; //Calculate and set the deadweight
			calculateInitial(dens); // calculate the value of the draught, lcf, lcb and mct for the given displacement
			
			//Updat the GUI
			Gui optiloading = Gui.getInstance();
			optiloading.misccompartments.loadTanks();
			optiloading.setConstants(displacement, deadweight);
			
			calculateBuoyancyDistribution(t,optiloading.misccompartments.getIntensityList()); //Calculate buoyancy distribution
			
			//Get the moments created by the miscellaneous compartments.
			ArrayList<Float> constantvalues = optiloading.misccompartments.getMoments();
			constlong = constantvalues.get(0);
			constvert = constantvalues.get(1);
			constfsm = constantvalues.get(2);
			
			optiloading = null; //release the GUI object
			validload = true;  // change the return value to true
		}
		return validload;
	}
	
	// Method to calculate the displacement at a desired TMD and water density
	private float getDispAtTMD(float tmd, float dens){
		float returnvalue = 0; //Initializing the return value
		float disp; //Initializing the displacement value
		for(int i = 0; i < data.size(); ++i) { //loop trough all the hydrostatic data
			HydrostaticData x = data.get(i); 
			if(tmd < x.draught) { // if the draught of the current HydrostaticData object is greater then we've gone past the wanted value
				HydrostaticData y = data.get(i-1); // get the previous object
				disp = interpolate(tmd,y.draught,x.draught,y.displacement,x.displacement); // interpolate to get the displacement at the wanted tmd
				returnvalue = (float) (disp * (dens/1.025)); // calculate the displacement for the set water density
				break;
			} else if(tmd == x.draught) { // if the draught is equal to the desired TMD than this is the object we want
				disp = x.displacement; // get the displacement of the object
				returnvalue = (float)  (disp * (dens/1.025)); //calculate the displacement for the set water density
				break;
			}
		}
		return returnvalue;
	}
	
	// Method to calculate the values of draught, lcf, lcb and mct
	private void calculateInitial(float dens){
		float tempdisplacement = (float) (displacement * (1.025/dens)); // convert the current displacement to the displacement in sea water (table values are for displacements in sea water)
		
		for(int i = 0; i < data.size(); ++i) { // loop trough all the hydrostatic data
			HydrostaticData x = data.get(i);
			if(tempdisplacement < x.displacement) { // if the displacement of the current object is greater then we've gone past the wanted value
				HydrostaticData y = data.get(i-1); // get the previous object
				
				// set values calculated with interpolation between the two objects
				draught = interpolate(displacement,y.displacement,x.displacement,y.draught,x.draught);
				lcf = interpolate(displacement,y.displacement,x.displacement,y.lcf,x.lcf);
				lcb = interpolate(displacement,y.displacement,x.displacement,y.lcb,x.lcb);
				km = interpolate(displacement,y.displacement,x.displacement,y.km,x.km);
				kb = interpolate(displacement,y.displacement,x.displacement,y.kb,x.kb);
				float tempmct = interpolate(displacement,y.displacement,x.displacement,y.mct,x.mct);
				mct = (float) (tempmct * (dens/1.025));
				break; // all values are set so we can break
			} else if(tempdisplacement == x.displacement) { // if the wanted displacement is equal to the displacement of the object we got the desired object
				
				// set all values
				draught = x.draught;
				lcf = x.lcf;
				lcb = x.lcb;
				km = x.km;
				kb = x.kb;
				float tempmct = x.mct;
				mct = (float) (tempmct * (dens/1.025));
				break; // all values are set so we can break
			}
		}
	}
	
	/**
	 * Method used to set Volumes and get the solution values (mainly used for testing purposes).
	 * @param volumes ArrayList<Float> containing the floating point values of each tank volume.
	 * @return Array of floating points containing the calculated values.
	 */
	public ArrayList<Float> getSolution(ArrayList<Float> volumes){
		// Setting the volume of each tank
		for(int i = 0; i < tanks.size(); ++i){
			Tank tank = tanks.get(i);
			tank.setVolume(volumes.get(i), toLoad.getSpecificgravity());
		}
		
		float templongmoment = 0f; // Initializing the temporary longitudinal moment
		float tempvertmoment = 0f; // Initializing the temporary vert moment
		float fsm = 0f; // Initializing the free surface moment
		
		for(Tank x: tanks){ // Run trough the tanks to get the moment eacht tank is creating
			templongmoment += (x.getLCG()*x.getCargo());
			tempvertmoment += (x.getKG()*x.getCargo());
			fsm += (x.getSpecificGravity()*x.getFSM());
		}
		
		// Add the light ship moment to get the total resulting moment
		templongmoment += (lightship*lightshipLCG);
		tempvertmoment += (lightship*lightshipKG);
		
		//Add the moments created by the miscellaneous tanks
		templongmoment += constlong;
		tempvertmoment += constvert;
		fsm += constfsm;
		
		lcg = templongmoment/displacement; // Calculate the longitudinal center of gravity
		kg = tempvertmoment/displacement; //Calculate the vertical center of gravity
		fsc = fsm/displacement; //Calculate the free surface moment
		gm = km - kg - fsc; //Calculate the metacentric height GM
		bm = km - kb; //Calculate the BM (used in the wall-sided formula)
		trim = ((lcb-lcg)*displacement)/(mct); //C alculate the trim
		daft = draught - ( (((lbpp/2)+lcg)*(-trim))/(lbpp)); // Calculate the draught aft
		dfwd = daft-trim; // Calculate the draught fwd
		
		//Return solutions
		ArrayList<Float> returnvalue = new ArrayList<Float>();
		returnvalue.add(trim);
		returnvalue.add(dfwd);
		returnvalue.add(daft);
		returnvalue.add(gm);
		returnvalue.add(fsc);
		returnvalue.add(bm);
		return returnvalue;
	}
	
	/**
	 * Method used to get ship particulars.
	 * @param particular ShipParticular indicating which particular to get (see enum class ShipParticular).
	 * @return Floating point value containing the wanted particular.
	 */
	public float getData(ShipParticular particular){
		float returnvalue = 0;
		
		//case statement on the enum ShipParticular
		switch(particular){
			case DISPLACEMENT: returnvalue = displacement; break;
			case MAXDISPLACEMENT: returnvalue = maxdisplacement ; break;
			case MINDISPLACEMENT: returnvalue = mindisplacement ; break;
			case LIGHTSHIP: returnvalue = lightship ; break;
			case LIGHTSHIP_LCF: returnvalue = lightshipLCG ; break;
			case LBPP: returnvalue = lbpp ; break;
			case NUMBEROFTANKS: returnvalue = tanks.size(); break;
			case DRAUGHT: returnvalue = draught; break;
			case LCF: returnvalue = lcf; break;
			case LCB: returnvalue = lcb; break;
			case LCG: returnvalue = lcg; break;
			case MCT: returnvalue = mct; break;
			case FSC: returnvalue = fsc; break;
			case KM: returnvalue = km; break;
			case KG: returnvalue = kg; break;
			case GM: returnvalue = gm; break;
			case KB: returnvalue = kb; break;
			case TRIM: returnvalue = trim; break;
			case DFWD: returnvalue = dfwd; break;
			case DAFT: returnvalue = daft; break;
			case DENSITY: returnvalue = density; break;
			case CONSTANTS: returnvalue = constants; break;
			case DEADWEIGHT: returnvalue = deadweight; break;
		}
		return returnvalue;
	}
	
	// Method used to interpolate between value
	// refnum is the number for which we want the value
	// refnum is between refsmall and reflarge which correspond to solsmall and solllargo
	private static float interpolate(float refnum, float refsmall, float reflarge, float solsmall, float sollarge){
		float x = refnum - refsmall;
		float y = reflarge - refsmall;
		float z = sollarge - solsmall;
		float solution = (x/y)*z;
		return (solution + solsmall);
	}
	
	// Inner class which is used to store hydrostatic data (each object corresponds with a row from the hydrostatic table)
	// These objects are stored in the ArrayList data in the Ship object
	private class HydrostaticData {

		// Different values from the hydrostatic table
		float displacement;
		float draught;
		float lcf;
		float lcb;
		float mct;
		float km;
		float kb;

		//Constructor used to set the values from the hydrostatic table
		HydrostaticData(float disp, float draft, float flo, float buo, float mom, float kme, float kbe){
			displacement = disp;
			draught = draft;
			lcf = flo;
			lcb = buo;
			mct = mom;
			km = kme;
			kb = kbe;
		}
	}

	// Inner class which is used to store section data (for buoyancy calculations)
	// These objects are stored in the ArrayList sectionlist
	private class Section {
		private float length;
		private float LCB;
		private float indication;
		private float percentage;
		private float p;

		//Constructor
		Section(float l, float lc){
			length = l;
			LCB = lc;
			indication = (float) (l/218.7);
		}

		//Get length of the section
		public float getLength(){
			return length;
		}

		//Get LCB of the section
		public float getLCB(){
			return LCB;
		}

		//Get indication of buoyancy force (equally divided between sections)
		public float getIndication(){
			return indication;
		}

		//Get the percentage of the total length
		public float getPercentage(){
			return percentage;
		}	
	}
}

